class EventType:
    MESSAGE = "MSG"
    MUTE = "MUTE"
    PRIVMSG = "PRIVMSG"
    PRIVMSGSENT = "PRIVMSGSENT"
    JOIN = "JOIN"
    QUIT = "QUIT"
    REFRESH = "REFRESH"
    BROADCAST = "BROADCAST"
    NAMES = "NAMES"
    ERROR = "ERR"
    BAN = "BAN"
    UNBAN = "UNBAN"
